package org.ow2.util.pool.impl.enhanced.callback;

import org.ow2.util.pool.impl.enhanced.api.basic.CreatePoolItemException;
import org.ow2.util.pool.impl.enhanced.api.basic.IPoolItemFactory;
import org.ow2.util.pool.impl.enhanced.impl.PoolError;
import org.ow2.util.pool.impl.enhanced.internal.stoppableexecution.IArgumentsCallable;
import org.ow2.util.pool.impl.enhanced.internal.stoppableexecution.StoppableExecution;
import org.ow2.util.pool.impl.enhanced.internal.stoppableexecution.StoppedExecutionException;

/**
 * @author Gael Lalire
 * @param <E>
 */
public class CreateCB<E> extends StoppableExecution<E> implements IPoolItemFactory<E> {

    public CreateCB(final IPoolItemFactory<E> create) {
        super(new IArgumentsCallable<E>() {

            public E call(final Object... args) throws Exception {
                return create.createPoolItem();
            }

        });
    }

    public E createPoolItem() throws CreatePoolItemException {
        try {
            return call();
        } catch (CreatePoolItemException e) {
            throw e;
        } catch (StoppedExecutionException e) {
            throw new CreatePoolItemException(null, e);
        } catch (RuntimeException e) {
            throw new CreatePoolItemException(null, e);
        } catch (Exception e) {
            throw new PoolError("createPoolItem could only throw CreatePoolItemException as checked exception", e);
        }
    }

}
