package org.ow2.util.pool.impl.enhanced.impl.keepbusy;

import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;

/**
 * @author Gael Lalire
 * @param <E> reference type
 */
public final class IdentityWeakReference<E> extends WeakReference<E> implements IIdentityReference<E> {

    private int hashCode;

    public IdentityWeakReference(final E reference, final ReferenceQueue<E> referenceQueue) {
        super(reference, referenceQueue);
        hashCode = System.identityHashCode(reference);
    }

    @Override
    public boolean equals(final Object object) {
        if (this == object) {
            return true;
        }
        if (!(object instanceof IIdentityReference<?>)) {
            return false;
        }
        Object reference = ((IIdentityReference<?>) object).getReference();
        Object localObject = get();
        if (localObject == null) {
            return false;
        }
        if (localObject != reference) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        return hashCode;
    }

    public E getReference() {
        return get();
    }
}
